/*
 * 				Twidere - Twitter client for Android
 * 
 *  Copyright (C) 2012-2014 Mariotaku Lee <mariotaku.lee@gmail.com>
 * 
 *  This program is free software: you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, either version 3 of the License, or
 *  (at your option) any later version.
 * 
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 * 
 *  You should have received a copy of the GNU General Public License
 *  along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.mariotaku.twidere.adapter

import android.content.Context
import android.view.View
import android.view.ViewGroup
import org.mariotaku.twidere.R
import org.mariotaku.twidere.adapter.iface.IBaseAdapter
import org.mariotaku.twidere.model.ParcelableUser
import org.mariotaku.twidere.model.UserKey
import org.mariotaku.twidere.util.Utils.configBaseAdapter
import org.mariotaku.twidere.util.Utils.getUserTypeIconRes
import org.mariotaku.twidere.view.holder.TwoLineWithIconViewHolder

class SimpleParcelableUsersAdapter @JvmOverloads constructor(
        context: Context,
        layoutRes: Int = R.layout.list_item_user
) : BaseArrayAdapter<ParcelableUser>(context, layoutRes), IBaseAdapter {

    init {
        configBaseAdapter(context, this)
    }

    override fun getItemId(position: Int): Long {
        val item = getItem(position)
        if (item != null) {
            return item.hashCode().toLong()
        } else {
            return -1
        }
    }

    override fun getView(position: Int, convertView: View?, parent: ViewGroup): View {
        val view = super.getView(position, convertView, parent)
        val tag = view.tag
        val holder: TwoLineWithIconViewHolder
        if (tag is TwoLineWithIconViewHolder) {
            holder = tag
        } else {
            holder = TwoLineWithIconViewHolder(view)
            view.tag = holder
        }

        val user = getItem(position)

        holder.text1.setCompoundDrawablesWithIntrinsicBounds(0, 0,
                getUserTypeIconRes(user.is_verified, user.is_protected), 0)
        holder.text1.text = userColorNameManager.getUserNickname(user.key, user.name)
        holder.text2.text = String.format("@%s", user.screen_name)
        holder.icon.visibility = if (isProfileImageDisplayed) View.VISIBLE else View.GONE
        if (isProfileImageDisplayed) {
            mediaLoader.displayProfileImage(holder.icon, user)
        } else {
            mediaLoader.cancelDisplayTask(holder.icon)
        }
        return view
    }

    @JvmOverloads fun setData(data: List<ParcelableUser>?, clearOld: Boolean = false) {
        if (clearOld) {
            clear()
        }
        if (data == null) return
        for (user in data) {
            if (clearOld || findUserPosition(user.key) < 0) {
                add(user)
            }
        }
    }

    fun findUserPosition(userKey: UserKey): Int {
        var i = 0
        val j = count
        while (i < j) {
            if (userKey == getItem(i).key) return i
            i++
        }
        return -1
    }

}
